'==========================================
' Table enums, structures, and messages
'==========================================
Enum TableSizeType
	Headers
	Visible
	All
End Enum

Enum PaintPart
	hdrAll
	selBack
	hotBack
	cellBack
	cellFore
End Enum

Enum HeaderStyle
	Themed
	Button
	Flat
End Enum

Enum TableCellStyle
	Normal
	Hot
	Pushed
	Inactive
	Selected
End Enum

Type CColor
	r As Byte
	g As Byte
	b As Byte
	a As Byte
End Type

Union CCOver
	p As CColor
	v As Dword
End Union

Type TableColumnData
	width As Long
	align As Dword
	foreColor As Dword: ' &hffffffff for default
	backColor As Dword: ' &hffffffff for default
End Type

Type ShiftKeys
	isShift As Long
	isAlt As Long
	isCtrl As Long
	isWnd As Long
End Type

Type DataSource
	getData As Dword: 'getData( ByVal r as Long, ByVal c as Long, ByRef value as WString ), row -1 = Header
	setData As Dword: 'setData( ByVal r as Long, ByVal c as Long, ByRef value as WString )
End Type

Type TableData
	rows As Long
	cols As Long
	drewHot As Long
	sizeCol As Long: ' Sizeable col, -1 for rowHeader, or -2 if not sizeable
	sizeLeft As Long: ' min x pos
	sizePos As Long: ' x position of mouse on mouse down
	sizing As Long
	allowResize As Long: ' -1 allow, 0 don't
	hSize As Dword: ' Size cursor
	hNorm As Dword: ' Normal cursor
	wasFocused As Long
	hotTrack As Long: ' -1/0 yes/no, hot track cell or row?
	topRow As Long
	leftCol As Long
	curCol As Long
	curRow As Long
	selectKey As Dword:     ' VK_x - specifies which key selects / send double click notify.  Defaults to VK_SPACE
	selectRow As Long:      ' -1/0
	horizontalGrid As Long: ' -1/0
	verticalGrid As Long:   ' -1/0
	rowHeight As Long
	frozRows As Long
	frozCols As Long
	hHeadFont As Dword
	hTextFont As Dword
	colHeaderStyle As Long: ' 1=themed, 2=button, 3=flat
	rowHeaderStyle As Long: ' 1=themed, 2=button, 3=flat
	showColHeaders As Long: ' -1/0
	showRowHeaders As Long: ' -1/0
	rowHeaderWidth As Long
	userPaint As Dword: ' User paint routine Function MyPaint( part as Long, state as Long, dta as DrawCellData )
	column As TableColumnData Ptr
	gridColor As Dword
	foreColor As Dword: ' Text color
	backColor As Dword: ' Background
	frozColor As Dword: ' Froz background
	workspaceColor As Dword: ' Non-grid area color
	selColor1 As Dword: ' Light gradient
	selColor2 As Dword: ' Dark gradient
	selBColor As Dword: ' Border color
	selTColor As Dword: ' Text Color
	hotColor1 As Dword: ' Light gradient
	hotColor2 As Dword: ' Dark gradient
	hotBColor As Dword: ' Border Color
	hotTColor As Dword: ' Text color
	altBack As Long:    ' Alternating back color
	alternateRows As Long: ' Alternate row backgrounds?

	sysGridColor As Long
	sysForeColor As Long: ' Using system text color?
	sysBackColor As Long: ' Using system background?
	sysFrozColor As Long: ' Using system froz color?
	sysWorkspaceColor As Long: ' Using system non-grid area color?
	sysSelColor1 As Long: ' Using system light gradient?
	sysSelColor2 As Long: ' Using system dark gradient?
	sysSelBColor As Long: ' Using system border color?
	sysSelTColor As Long: ' Using system text Color?
	sysHotColor1 As Long: ' Using system light gradient?
	sysHotColor2 As Long: ' Using system dark gradient?
	sysHotBColor As Long: ' Using system border Color?
	sysHotTColor As Long: ' Using system text color?
	sysAltBack As Long:   ' Using system alternating back color?

	ds As DataSource
End Type

Type DrawCellData
	r As Long
	c As Long
	hWnd As Dword
	value As WStringZ Ptr
	hdc As Dword
	rc As RECT
	cursor As Point
	drawStyle As Long
	align As Dword
	headerStyle As Long: ' 1=themed, 2=button, 3=flat
	hFont As Dword
	foreColor As Dword: ' Cell drawing/text color
	backColor As Dword: ' Cell back color
	selColor1 As Dword: ' Light gradient
	selColor2 As Dword: ' Dark gradient
	selBColor As Dword: ' Border color
	selTColor As Dword: ' Text Color
	hotColor1 As Dword: ' Light gradient
	hotColor2 As Dword: ' Dark gradient
	hotBColor As Dword: ' Border Color
	hotTColor As Dword: ' Text color
	altBack As Long:    ' Alternating back color
	selectRow As Long
	table As TableData Ptr
End Type

' Table Methods
'User must define these and set the datasource
'Sub Test_GetData( ByVal r As Long, ByVal c As Long, ByRef value As WString )

'Optional
'Sub Test_SetData( ByVal r As Long, ByVal c As Long, ByRef value As WString )
'Function Test_GridPaint( ByVal part As Long, ByVal drwState As Long, ByRef value As WString, ByRef dta As DrawCellData ) As Long
'  The paint routine allows customization two ways.  If you want to draw the part, draw it and return true, if you
'  want to alter values in drawcelldata you can do that and return 0/false.

'Supplied Table functions
'Function NewTable Alias "NewTable"(ByVal hParent As Long, ByVal pLeft As Long, ByVal pTop As Long, ByVal pWidth As Long, ByVal pHeight As Long, ByVal Id As Long) Export Common As Long
'Sub Table_SetSelectKey( ByVal hWnd As Dword, ByVal vkKey As Dword ): ' Use -1 to clear select key
'Sub Table_SetUserPaint( ByVal hWnd As Dword, procedure As Dword )
'Sub Table_SetHeaders( ByVal hWnd As Dword, ByVal showColHeaders As Long, ByVal showRowHeaders As Long, ByVal rowHeaderWidth As Long )
'Sub Table_SetFrozen( ByVal hWnd As Dword, ByVal rows As Long, ByVal cols As Long, ByVal clr As Dword ) Export Common
'Sub Table_SetDataSource2( ByVal hWnd As Dword, ByVal getData As Dword, ByVal setData As Dword ) Export Common
'Sub Table_SetDataSource( ByVal hWnd As Dword, ByRef ds As DataSource ) Export Common
'Sub Table_SetStyle( ByVal hWnd As Dword, ByVal colStyle As Long, ByVal rowStyle As Long ) Export Common
'Sub Table_SetRows( ByVal hWnd As Dword, ByVal value As Long ) Export Common
'Sub Table_SetCols( ByVal hWnd As Dword, ByVal value As Long ) Export Common
'Sub Table_SetCol( ByVal hWnd As Dword, ByVal c As Long, ByVal w As Long, ByVal align As Dword, ByVal foreColor As Dword, ByVal backColor As Dword ) Export Common
'Sub Table_SetPos( ByVal hWnd As Dword, ByVal r As Long, ByVal c As Long ) Export Common
'Sub Table_SetSelectRow( ByVal hWnd As Dword, ByVal value As Long ) Export Common
'Sub Table_SetTablelines( ByVal hWnd As Dword, ByVal showHoriz As Long, ByVal showVert As Long, ByVal gridColor As Dword ) Export Common
'Sub Table_SetRowHeight( ByVal hWnd As Dword, ByVal value As Long ) Export Common
'Sub Table_SetForeColor( ByVal hWnd As Dword, ByVal value As Dword ) Export Common
'Sub Table_SetBackColor( ByVal hWnd As Dword, ByVal value As Dword ) Export Common
'Sub Table_SetAlternatingRows( ByVal hWnd As Dword, ByVal value As Long, ByVal c As Dword ) Export Common
'Sub Table_SetWorkspaceColor( ByVal hWnd As Dword, ByVal value As Dword ) Export Common
'Sub Table_SetSelectColors( ByVal hWnd As Dword, ByVal color1 As Dword, ByVal color2 As Dword, ByVal borderColor As Dword, ByVal textColor As Dword ) Export Common
'Sub Table_SetHotColors( ByVal hWnd As Dword, ByVal color1 As Dword, ByVal color2 As Dword, ByVal borderColor As Dword, ByVal textColor As Dword ) Export Common
'
'
'Sub Table_GetSelectKey( ByVal hWnd As Dword, ByRef vkKey As Dword )
'Sub Table_GetHeaders( ByVal hWnd As Dword, ByRef showColHeaders As Long, ByRef showRowHeaders As Long, ByRef rowHeaderWidth As Long )
'Sub Table_GetFrozen( ByVal hWnd As Dword, ByRef rows As Long, ByRef cols As Long, ByRef clr As Dword ) Export Common
'Sub Table_GetDataSource( ByVal hWnd As Dword, ByRef ds As DataSource ) Export Common
'Sub Table_GetStyle( ByVal hWnd As Dword, ByRef colStyle As Long, ByRef rowStyle As Long ) Export Common
'Sub Table_GetRows( ByVal hWnd As Dword, ByRef value As Long ) Export Common
'Sub Table_GetCols( ByVal hWnd As Dword, ByRef value As Long ) Export Common
'Sub Table_GetCol( ByVal hWnd As Dword, ByVal c As Long, ByRef w As Long, ByRef align As Dword, ByRef foreColor As Dword, backColor As Dword ) Export Common
'Sub Table_GetPos( ByVal hWnd As Dword, ByRef r As Long, ByRef c As Long ) Export Common
'Sub Table_GetSelectRow( ByVal hWnd As Dword, ByRef value As Long ) Export Common
'Sub Table_GetTable( ByVal hWnd As Dword, ByRef showHoriz As Long, ByRef showVert As Long, ByRef gridColor As Dword ) Export Common
'Sub Table_GetRowHeight( ByVal hWnd As Dword, ByRef value As Long ) Export Common
'Sub Table_GetForeColor( ByVal hWnd As Dword, ByRef value As Dword ) Export Common
'Sub Table_GetBackColor( ByVal hWnd As Dword, ByRef value As Dword ) Export Common
'Sub Table_GetAlternatingRows( ByVal hWnd As Dword, ByRef value As Long, ByRef c As Dword ) Export Common
'Sub Table_GetWorkspaceColor( ByVal hWnd As Dword, ByRef value As Dword ) Export Common
'Sub Table_GetSelectColors( ByVal hWnd As Dword, ByRef color1 As Dword, ByRef color2 As Dword, ByRef borderColor As Dword, ByRef textColor As Dword ) Export Common
'Sub Table_GetHotColors( ByVal hWnd As Dword, ByRef color1 As Dword, ByRef color2 As Dword, ByRef borderColor As Dword, ByRef textColor As Dword ) Export Common

' WM_NOTIFY message sent to parent on key select or mouse double click
Type TblNotify
	hdr As NmHdr
	msg As Dword
	r As Long
	c As Long
	dta As TableData Ptr
End Type

%TBN_Select = 1
%TBN_CellChg = 2

'Example notify code for WM_NOTIFY, lParam points to NMHDR / TblNotify
'Local ptbl As TblNotify Ptr
'If @hPtr.idFrom = %CTL_TABLE And @hPtr.code = %TBN_Select Then
'  ptbl = hPtr
'  SetWindowText hWNd, "Selected " + Format$( @ptbl.r + 1, "#,##0" ) + " : " + Format$( @ptbl.c+1, "#,##0" )
'End If

'Example user paint
'Function Test_GridPaint( ByVal part As Long, ByVal drwState As Long, ByRef value As WString, ByRef dta As DrawCellData ) As Long
'  Function = 0: ' Everything else we didn't draw
'  If part = %PaintPart.cellFore And dta.r=2 And dta.c=2 Then
'    DrawText( dta.hdc, "OMG", 3, dta.rc, dta.align )
'    Function = -1: ' Since we drew the foreground, say we did
'  End If
'End Function
